﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FYEDrawDemo_Senior
{
    public partial class MainForm : Form
    {
        private FYECMDCView cmdcView = null;
        private StatusCfg staCfg = null;


        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            btnLoadStatusCfg.Enabled = false;
            btnStatusUpdate.Enabled = false;
            btnLoadCompCfg.Enabled = false;
            btnCompUpdate.Enabled = false;
        }

        private void MainForm_SizeChanged(object sender, EventArgs e)
        {
            this.panel1.Width = this.ClientSize.Width - 26;
            this.panel1.Height = this.ClientSize.Height - 100;
            if (cmdcView != null)
            {
                this.cmdcView.SetRect(this.panel1.Width, this.panel1.Height);
            }

        }

        private void btnLoadStatusMap_Click(object sender, EventArgs e)
        {
            btnLoadStatusCfg.Enabled = true;
            btnStatusUpdate.Enabled = false;
            btnLoadCompCfg.Enabled = false;
            btnCompUpdate.Enabled = false;

            if (cmdcView != null)
            {
                cmdcView.Release();
                cmdcView = null;
            }
            cmdcView = new FYECMDCView();
            cmdcView.Create(this.panel1);
            String stationFileName = System.Environment.CurrentDirectory + "\\2 火车站CTC状态图.txt";
            cmdcView.Open(stationFileName);
            //cmdcView.SetBackColor(0x0);
            //cmdcView.SetNoClientColor(0x0);
            cmdcView.Load();                  //读取绘图数据物理图元，生成逻辑图元，进而与业务对象关联上
            cmdcView.SetRcvMsgWnd(this);      //如果不接收绘图区的鼠标消息，则不需要此行
        }
        private void btnGatherStatusMap_Click(object sender, EventArgs e)
        {
            btnLoadStatusCfg.Enabled = true;
            btnStatusUpdate.Enabled = false;
            btnLoadCompCfg.Enabled = false;
            btnCompUpdate.Enabled = false;

            if (cmdcView != null)
            {
                cmdcView.Release();
                cmdcView = null;
            }
            cmdcView = new FYECMDCView();
            cmdcView.Create(this.panel1);
            String stationFileName = System.Environment.CurrentDirectory + "\\1 采集设备状态图.txt";
            cmdcView.Open(stationFileName);
            //cmdcView.SetBackColor(0x0);
            //cmdcView.SetNoClientColor(0x0);
            cmdcView.Load();
            cmdcView.SetRcvMsgWnd(this);      //如果不接收绘图区的鼠标消息，则不需要此行
        }

        private void btnLoadStatusCfg_Click(object sender, EventArgs e)
        {
            btnStatusUpdate.Enabled = true;

            if(staCfg != null)
            {
                staCfg.Clear();
                staCfg = null;
            }
            staCfg = new StatusCfg();
            staCfg.Load("状态图配置示例.txt");
        }

        private void btnStatusUpdate_Click(object sender, EventArgs e)
        {
            string sText = btnStatusUpdate.Text;
            if (sText.Contains("启动"))
            {
                this.timer1.Start();
                btnStatusUpdate.Text = "4停止";
            }
            else if(sText.Contains("停止"))
            {
                this.timer1.Stop();
                btnStatusUpdate.Text = "3启动定时更新";
            }
        }

        private void btnLoadCompMap_Click(object sender, EventArgs e)
        {
            btnLoadStatusCfg.Enabled = false;
            btnStatusUpdate.Enabled = false;
            btnLoadCompCfg.Enabled = true;
            btnCompUpdate.Enabled = false;

            if (cmdcView != null)
            {
                cmdcView.Release();
                cmdcView = null;
            }
            cmdcView = new FYECMDCView();
            cmdcView.Create(this.panel1);
            String stationFileName = System.Environment.CurrentDirectory + "\\11 25相敏轨道电路图.txt";
            cmdcView.Open(stationFileName);
            //cmdcView.SetBackColor(0x0);
            //cmdcView.SetNoClientColor(0x0);
            cmdcView.Load();
            //cmdcView.SetRcvMsgWnd(this);      //如果不接收绘图区的鼠标消息，则不需要此行
        }

        private void btnLoadCompCfg_Click(object sender, EventArgs e)
        {
            btnCompUpdate.Enabled = true;

            if (staCfg != null)
            {
                staCfg.Clear();
                staCfg = null;
            }
            staCfg = new StatusCfg();
            staCfg.Load("复合图元状态配置通用.txt");
        }

        private void btnCompUpdate_Click(object sender, EventArgs e)
        {
            string sText = btnCompUpdate.Text;
            if (sText.Contains("启动"))
            {
                this.timer1.Start();
                btnCompUpdate.Text = "4停止";
            }
            else if (sText.Contains("停止"))
            {
                this.timer1.Stop();
                btnCompUpdate.Text = "3启动定时更新";
            }
        }


        private void chkEditMode_CheckedChanged(object sender, EventArgs e)
        {
            if (cmdcView != null)
            {
                cmdcView.SetEditMode(chkEditMode.Checked);
            }
        }
        
        private Random rand = new Random();
        private void timer1_Tick(object sender, EventArgs e)
        {
            //实际情况是有一套业务系统和绘图系统，本项目是绘图系统。画的图元都是业务系统存在的业务对象，实时状态数据也是来自业务系统。
            //此Demo不构建完整的业务系统，直接随机改变绘图系统状态值，从而更新绘图的线条、区域、文字等展示效果
            List<ShapeElement> shapes = cmdcView.GetShapeList();
            List<BaseStatus> statuss = staCfg.StatusList;
            foreach (ShapeElement shape in shapes)
            {
                foreach(BaseStatus sta in statuss)
                {
                    if(shape.Title == sta.TypeName)
                    {
                        //实际情况浮云绘图编辑器画图配置的状态数据是由业务系统的业务对象状态数决定的
                        //此Demo仅演示绘图系统不构建完整的业务系统，此处的图元实时状态值直接从图元可用状态数随机
                        int curStatus = rand.Next(sta.StatusCount);
                        if(curStatus != shape.Status)
                        {
                            shape.Status = curStatus;
                            staCfg.SetStatus(cmdcView, shape, curStatus);
                        }
                    }
                }
            }
        }
    }
}
