﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace FYEDrawDemo_Senior
{
    public abstract class BaseStatus 
    {
        public String TypeName { set; get; }
        public int StatusCount { set; get; }
        public abstract void AddStatus(string sParam, char separator);

        public abstract void SetStatus(FYECMDCView dcView, ShapeElement shape, int status);

        public abstract void Clear();
    }
    public class BaseParam { }
    public class LineParam : BaseParam
    {
        public int Color { set; get; }
        public int Width { set; get; }
        public int Style { set; get; }

        public LineParam(int color, int width, int style)
        {
            this.Color = color;
            this.Width = width;
            this.Style = style;
        }
    }
    public class AreaParam : BaseParam
    {
        public int Fill { set; get; }
        public int Color { set; get; }

        public AreaParam(int fill, int color)
        {
            this.Fill = fill;
            this.Color = color;
        }
    }
    public class CompParam : BaseParam
    {
        public int LineColor { set; get; }
        public int LineWidth { set; get; }
        public int LineStyle { set; get; }
        public int FillColor { set; get; }

        public CompParam(int lineColor,int lineWidth, int lineStyle, int fillColor)
        {
            this.LineColor = lineColor;
            this.LineWidth = lineWidth;
            this.LineStyle = lineStyle;
            this.FillColor = fillColor;
        }
    }

    public class LineStatus : BaseStatus
    {
        private List<LineParam> staList = new List<LineParam>();
        public List<LineParam> StatusList { get { return staList; } }

        public override void AddStatus(string sParam, char separator)
        {//状态0颜色值,线宽,虚实
            int color;
            int width;
            int style;
            string[] arr = sParam.Split(separator);
            Debug.Assert(arr.Length == 3);

            color = int.Parse(arr[0]);
            width = int.Parse(arr[1]);
            style = int.Parse(arr[2]);

            LineParam param = new LineParam(color, width, style);
            staList.Add(param);

        }

        public override void SetStatus(FYECMDCView dcView, ShapeElement shape, int status)
        {
            dcView.SetLineColor(shape, staList[status].Color);
            dcView.SetLineWidth(shape, staList[status].Width);
            dcView.SetLineStyle(shape, staList[status].Style);
        }

        public override void Clear()
        {
            staList.Clear();
        }
    }
    public class AreaStatus : BaseStatus
    {
        private List<AreaParam> staList = new List<AreaParam>();
        public List<AreaParam> StatusList { get { return staList; } }

        public override void AddStatus(string sParam, char separator)
        {//状态0是否填充,填充颜色
            int fill;
            int color;    
            string[] arr = sParam.Split(separator);
            Debug.Assert(arr.Length == 2);

            fill = int.Parse(arr[0]);
            color = int.Parse(arr[1]);

            AreaParam param = new AreaParam(fill, color);
            staList.Add(param);
        }

        public override void SetStatus(FYECMDCView dcView, ShapeElement shape, int status)
        {
            dcView.SetIsFill(shape, staList[status].Fill);
            dcView.SetFillColor(shape, staList[status].Color);
        }

        public override void Clear()
        {
            staList.Clear();
        }
    }

    public class CompStatus:BaseStatus
    {
        private List<CompParam> staList = new List<CompParam>();
        public List<CompParam> StatusList { get { return staList; } }

        public override void AddStatus(string sParam, char separator)
        {//状态0是否填充,填充颜色
            int lColor;
            int lWidth;
            int lStyle;
            int fColor;
            string[] arr = sParam.Split(separator);
            Debug.Assert(arr.Length == 4);

            lColor = int.Parse(arr[0]);
            lWidth = int.Parse(arr[1]);
            lStyle = int.Parse(arr[2]);
            fColor = int.Parse(arr[3]);

            CompParam param = new CompParam(lColor,lWidth,lStyle, fColor);
            staList.Add(param);
        }

        public override void SetStatus(FYECMDCView dcView, ShapeElement shape, int status)
        {
            dcView.SetLineColor(shape, staList[status].LineColor);
            dcView.SetLineWidth(shape, staList[status].LineWidth);
            dcView.SetLineStyle(shape, staList[status].LineStyle);
            dcView.SetFillColor(shape, staList[status].FillColor);
        }

        public override void Clear()
        {
            staList.Clear();
        }
    }
    public class StatusCfg
    {
        private List<BaseStatus> staList = new List<BaseStatus>();
        public List<BaseStatus> StatusList { get { return staList; } }

        public void Load(string fileName)
        {
            var file = File.Open(Environment.CurrentDirectory + "\\"+ fileName, FileMode.Open);
            List<string> txtRows = new List<string>();
            using (var stream = new StreamReader(file,System.Text.Encoding.GetEncoding("utf-8")))
            {
                while(!stream.EndOfStream)
                {
                    txtRows.Add(stream.ReadLine());
                }
            }
            file.Close();

            //;;线类型名:状态数N;状态0颜色值,线宽,虚实;状态1颜色值,线宽,虚实......状态N颜色值,线宽,虚实
            //LT1:3;12632256,1,0;65280,1,0;255,1,0
            //LT2:5;12632256,1,1;65280,2,0;255,2,0;16711680,2,0;16711935,1,0 

            //;;块类型名:状态数N;状态0是否填充,填充颜色;状态1是否填充,填充颜色......状态N是否填充,填充颜色
            //AT1:2;0,0;1,65280
            //AT2:5;0,0;1,65280;1,255;116711680;16711935,1

            //;;类型名称: 状态数;状态0线颜色,线宽度,填充颜色;状态1线颜色,线宽度,填充颜色......
            //CMT1:3;12632256,1,0;65280,1,0,0,0;255,1,0
            //CMT2:5;12632256,1,0;65280,1,0;255,1,0;16711680,2,12632256;16711935,2,65280
            foreach (string row in txtRows)
            {
                if (row.Length < 2 || row.StartsWith(";;")) continue;

                string[] str1 = row.Split(':');
                string typeName = str1[0];
                string str2 = str1[1];

                string[] str3 = str2.Split(";");
                int count = int.Parse(str3[0]);
                Debug.Assert(str3.Length>=count+1);

                BaseStatus sta = null;
                if(typeName.Contains("LT"))
                {
                    sta = new LineStatus();
                }
                else if(typeName.Contains("AT"))
                {
                    sta = new AreaStatus();
                }
                else if(typeName.Contains("CMT"))
                {
                    sta = new CompStatus();
                }
                for (int i = 0; i < count; i++)
                {
                    string sp = str3[1 + i];
                    sta.AddStatus(sp, ',');
                }
                sta.TypeName = typeName;
                sta.StatusCount = count;
                staList.Add(sta);
            }
        }

        public void SetStatus(FYECMDCView dcView, ShapeElement shape, int status)
        {
            foreach(BaseStatus sta in staList)
            {
                if(sta.TypeName == shape.Title)
                {
                    sta.SetStatus(dcView, shape, status);
                }
            }
        }

        public void Clear()
        {
            foreach(BaseStatus bs in staList)
            {
                bs.Clear();
            }
            staList.Clear();
        }
    }
}
